/* ////////////////////////////////////////////////////////////////////////////

// Project: Sharing data from a RDBMS with a Protege project using JDBC/ODBC.

// Developer: My Nguyen
// Developer: Adam Silberfein (2005 updates)

/////////////////////////////////////////////////////////////////////////////*/

package DataGenieTab;

///////////////////////////////////////////////////////////////////////////////
// IMPORTS

// Java SDK
import java.lang.*;

// For the Swing UI Components
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.border.*;

// For the JDBC
import java.sql.*;

// For Vector and Collections classes
import java.util.*;

// For File I/O
import java.io.*;

// For the Protege PDK
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;


///////////////////////////////////////////////////////////////////////////////
// ConnectInfoPanel

// ConnectInfoPanel class encapsulates the GUI components that let the user
// connect to a data source, see the tables stored there, and import the data
// into the current Protege project.

class ConnectInfoPanel extends JPanel
    implements ActionListener, TreeSelectionListener, ItemListener {

	// Constructor

    public ConnectInfoPanel(DataPreviewPanel previewPanel, KnowledgeBase kb) {
		// retrieve the list of stored data source names
		Vector dsnList = new Vector();
		try {
			File file = new File(Global.tabName);
			if (file.exists()) {
				FileReader fileReader = new FileReader(file);
				BufferedReader reader = new BufferedReader(fileReader);
				
				String line = reader.readLine();
				while (line != null) {
					dsnList.addElement(line);
					line = reader.readLine();
				}

				reader.close();
				fileReader.close();
			} else {
				Global.debug(Global.tabName + " file does not exist.");
			}
		} catch (NullPointerException ex) {
			ex.printStackTrace();
		} catch (FileNotFoundException ex) {
			ex.printStackTrace();
		} catch (IOException ex) {
			ex.printStackTrace();
		}

        // initialize
        this.previewPanel = previewPanel;
        this.kb = kb;

		tables = new Vector();
        
        // add UI components
        setLayout(new BorderLayout());
        
        // NORTH
        JPanel pNorth = new JPanel(new GridLayout(6, 1));

        JPanel p0 = new JPanel(new FlowLayout(FlowLayout.LEFT));
        p0.add(new JLabel("Data Source Type", JLabel.LEFT));
        optODBC = new JRadioButton("ODBC", true);
        optODBC.setActionCommand("optODBC");
        optODBC.addActionListener(this);
        optJDBC = new JRadioButton("JDBC", false);
        optJDBC.setActionCommand("optJDBC");
        optJDBC.addActionListener(this);
        ButtonGroup dataGroup = new ButtonGroup();
        dataGroup.add(optODBC);
        dataGroup.add(optJDBC);
        p0.add(optODBC);
        p0.add(optJDBC);
        pNorth.add(p0);

        JPanel p1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
        p1.add(driverLabel);
        txtDriver.setText(strODBCDriver);
		txtDriver.setEditable(false);
        p1.add(txtDriver);
        pNorth.add(p1);
        
        JPanel p2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
        p2.add(dsLabel);
		cmbDSN = new JComboBox(dsnList);
        cmbDSN.setEditable(true);
        p2.add(cmbDSN);
        pNorth.add(p2);
        
        JPanel p3 = new JPanel(new FlowLayout(FlowLayout.LEFT));
        p3.add(new JLabel("User Login", JLabel.LEFT));
        p3.add(txtUserID);
        pNorth.add(p3);
        
        JPanel p4 = new JPanel(new FlowLayout(FlowLayout.LEFT)); 
        p4.add(new JLabel("Password", JLabel.LEFT));
        p4.add(txtPW);
        pNorth.add(p4);
        
        JPanel p5 = new JPanel();
		btnConnect.setMnemonic('C');
        btnConnect.addActionListener(this);
        p5.add(btnConnect);
		infoButton.setMnemonic('F');
		infoButton.addActionListener(this);
		p5.add(infoButton);
        pNorth.add(p5);
        add(pNorth,BorderLayout.NORTH);        
        
        // CENTER
        JPanel p6 = new JPanel(new BorderLayout());
		JPanel tableTypesPanel = new JPanel(new FlowLayout());
		sysCheckBox = new JCheckBox("System Table");
		sysCheckBox.setMnemonic('y');
		sysCheckBox.addItemListener(this);
		tableTypesPanel.add(sysCheckBox);
		tableCheckBox = new JCheckBox("Table");
		tableCheckBox.setMnemonic('t');
		tableCheckBox.addItemListener(this);
		tableTypesPanel.add(tableCheckBox);
		viewCheckBox = new JCheckBox("View");
		viewCheckBox.setMnemonic('V');
		viewCheckBox.addItemListener(this);
		tableTypesPanel.add(viewCheckBox);

		/*
		tableTypesPanel.add(new JCheckBox("Synonym"));
		tableTypesPanel.add(new JCheckBox("Alias"));
		tableTypesPanel.add(new JCheckBox("Global Temporary"));
		tableTypesPanel.add(new JCheckBox("Local Temporary"));
		*/
		otherCheckBox = new JCheckBox("Other");
		otherCheckBox.setMnemonic('O');
		otherCheckBox.addItemListener(this);
		tableTypesPanel.add(otherCheckBox);
		p6.add(new LabeledComponent("Data Tables",tableTypesPanel,false),BorderLayout.NORTH);
		treeTables.setCellRenderer(new MyTreeCellRenderer());
        treeTables.setEditable(false);
		treeTables.setRootVisible(false);
		treeTables.setShowsRootHandles(true);
        treeTables.putClientProperty("JTree.lineStyle", "Angled");
        treeTables.addTreeSelectionListener(this);
        JScrollPane scrollPane = new JScrollPane(treeTables);
        scrollPane.setPreferredSize(new Dimension(350, 300));
        //scrollPane.getViewport().add(treeTables);
        //p6.add(new LabeledComponent("Data Tables",scrollPane,true),BorderLayout.CENTER);
		p6.add(scrollPane,BorderLayout.CENTER);
        add(p6,BorderLayout.CENTER);
        
        // SOUTH
        JPanel pSouth = new JPanel(new BorderLayout());

        optEntireDB = new JRadioButton("Straight-thru dump - all tables above",false);
        optEntireDB.setActionCommand("optEntireDB");
		optEntireDB.setMnemonic('A');
		optEntireDB.setEnabled(false);
        optEntireDB.addActionListener(this);
        optSelected = new JRadioButton("Straight-thru dump - selected tables",true);
        optSelected.setActionCommand("optSelected");
		optSelected.setMnemonic('S');
		optSelected.setEnabled(false);
        optSelected.addActionListener(this);
        optHeuristic = new JRadioButton("Heuristic",false);
        optHeuristic.setActionCommand("optHeuristic");
		optHeuristic.setMnemonic('H');
		optHeuristic.setEnabled(false);
        optHeuristic.addActionListener(this);
        ButtonGroup methodsGroup = new ButtonGroup();
        methodsGroup.add(optEntireDB);
        methodsGroup.add(optSelected);
        methodsGroup.add(optHeuristic);
		Border etched = BorderFactory.createEtchedBorder();
		Border titled = BorderFactory.createTitledBorder(etched, "Import Method");
		JPanel p6a = new JPanel(new GridLayout(3, 1));
		p6a.setBorder(titled);
		p6a.add(optEntireDB);
		p6a.add(optSelected);
		p6a.add(optHeuristic);
		pSouth.add(p6a,BorderLayout.CENTER);
        
        JPanel p7 = new JPanel();
		btnDumpToProtegeProject.setMnemonic('I');
		btnDumpToProtegeProject.setEnabled(false);
        btnDumpToProtegeProject.addActionListener(this);
        p7.add(btnDumpToProtegeProject);
        pSouth.add(p7,BorderLayout.SOUTH);
        
        add(pSouth,BorderLayout.SOUTH);

		connected = false;
		setConnectedGUI(false);
    }
    
    // Handle action events, such as those from the connect and import
	// buttons
	
	public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == btnConnect) {
			// reset data and GUI components
            previewPanel.deleteTable();
			tables.removeAllElements();			
            rootNode = new DefaultMutableTreeNode(new TreeNodeInfo("Data Source",TreeNodeInfo.OTHER));
            treeModel = new DefaultTreeModel(rootNode);
            treeTables.setModel(treeModel);
			nImportMethod = IMPORT_SELECTED;
			optSelected.doClick();
			setConnectedGUI(false);

            try {
                
	            String strURL = "";
				Connection con = null;
				Class.forName(txtDriver.getText());
				
				if(nDSType == ODBC)
					strURL = "jdbc:odbc:" + (String)cmbDSN.getSelectedItem(); 
				else 
					strURL = (String)cmbDSN.getSelectedItem(); 
				
				con = DriverManager.getConnection(strURL, txtUserID.getText(), new String(txtPW.getPassword()));

                    
                DatabaseMetaData dbMetaData = con.getMetaData();

		        if (connectInfo == null)
					connectInfo = new ConnectInfo(txtDriver.getText(), strURL, txtUserID.getText(), new String(txtPW.getPassword()));
				else
					connectInfo.set(txtDriver.getText(), strURL, txtUserID.getText(), new String(txtPW.getPassword()));

				dsInfo = new DataSourceInfo(connectInfo);
				
				// Obtain all the available table types from the data source
				ResultSet rs = dbMetaData.getTableTypes();
				Vector tableTypesVector = new Vector();
				boolean more = rs.next();
				while (more) {
					String type = rs.getString("TABLE_TYPE");
					if(type.equals("TABLE") || type.equals("VIEW"))					
						tableTypesVector.addElement(type);
					more = rs.next();
				}
				rs.close();

				int numTypes = tableTypesVector.size();
				String[] tableTypes = new String[numTypes];
				for (int i = 0; i < numTypes; i++)
					tableTypes[i] = (String)tableTypesVector.get(i);

				// Set up the data tables tree				
				ResultSet rsTables = dbMetaData.getTables(null, null, null, tableTypes);
		        boolean bMore = rsTables.next();
		        while (bMore) {
				    String strTableName = rsTables.getString("TABLE_NAME");
System.out.println("---Examining table: " + strTableName);
					String tableTypeStr = rsTables.getString("TABLE_TYPE");
					int tableType;
					if (tableTypeStr.compareToIgnoreCase("SYSTEM TABLE") == 0)
						tableType = TreeNodeInfo.TABLE_SYSTEM;
					else if (tableTypeStr.compareToIgnoreCase("TABLE") == 0)
						tableType = TreeNodeInfo.TABLE;
					else if (tableTypeStr.compareToIgnoreCase("VIEW") == 0)
						tableType = TreeNodeInfo.TABLE_VIEW;
					else
						tableType = TreeNodeInfo.TABLE_OTHER;						
    			    
					// Get primary keys
					ArrayList primaryKeys = new ArrayList();
					ArrayList foreignKeys = new ArrayList();					

					try {
						rs = dbMetaData.getPrimaryKeys(null,null,strTableName);
						more = rs.next();
						while (more) {
							primaryKeys.add(rs.getString("COLUMN_NAME"));
							more = rs.next();
						}
						rs.close();

						rs = dbMetaData.getImportedKeys(null, null, strTableName);
						while(rs.next()) {
							String localField = rs.getString("PKCOLUMN_NAME");
							String refField = rs.getString("FKCOLUMN_NAME");
							String refTable = rs.getString("PKTABLE_NAME");
							String fkName = "fk_" + localField + "_" + refTable + "_" + refField;
							ForeignKey fk = new ForeignKey(fkName, localField, refField, refTable);
							foreignKeys.add(fk);
						}
						
					} catch (Exception sqlex) { System.out.println("QUERYING A TABLE FOR FOREIGN KEYS IS NOT SUPPORTED BY THIS DRIVER"); }
    			    
			        TreeNodeInfo nodeInfo = new TreeNodeInfo(strTableName,tableType);
			        nodeInfo.primaryKeyFields = primaryKeys;
			        nodeInfo.foreignKeys = foreignKeys;
	
					DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(nodeInfo);
			        treeModel.insertNodeInto(childNode, rootNode, rootNode.getChildCount());
			        treeTables.scrollPathToVisible(new TreePath(childNode.getPath()));
			        ResultSet rsColumns = dbMetaData.getColumns(null,null,strTableName,null);
			        boolean bMore2 = rsColumns.next();
			        while (bMore2) {
			            String strColumnName = rsColumns.getString("COLUMN_NAME");
			            String strTypeName = rsColumns.getString("TYPE_NAME");
						int sqlType = rsColumns.getShort("DATA_TYPE");
						ValueType protegeType = Global.getSQLProtegeType(sqlType);
						String nodeText = strColumnName+" ("+strTypeName+" --> "+protegeType.toString()+")";

						// Determine whether this column is a primary key
						int colType = TreeNodeInfo.COLUMN;
						for (int i = 0; i < primaryKeys.size(); i++) {
							String key = (String)primaryKeys.get(i);
							if (key.compareToIgnoreCase(strColumnName) == 0) {
								colType = TreeNodeInfo.COLUMN_PK;
								break;
							}
						}

						TreeNodeInfo nodeInfo2 = new TreeNodeInfo(nodeText,colType);
						DefaultMutableTreeNode childColumnNode = new DefaultMutableTreeNode(nodeInfo2);
			            treeModel.insertNodeInto(childColumnNode, childNode, childNode.getChildCount());
    		        
			            bMore2 = rsColumns.next();
			        }
			        rsColumns.close();
    			    
			        tables.add(nodeInfo);
					
					bMore = rsTables.next();
		        }
    		    
		        rsTables.close();
		        con.close();
		        
		        // Save this data source to the data source list
				String selectedItemText = (String)cmbDSN.getSelectedItem();
		        int nCount = cmbDSN.getItemCount();
		        int i = 0;
		        while (i < nCount) {
		            String str = (String)cmbDSN.getItemAt(i);
		            if (str.equalsIgnoreCase(selectedItemText))
		                break;
		            i++;
		        }
		        
		        if ((i == nCount) || (nCount == 0))
		            cmbDSN.addItem(selectedItemText);

				// Set the table type check boxes according to the types
				// available in the data source
				if (tables.size() == 0)
					JOptionPane.showMessageDialog(JOptionPane.getFrameForComponent(this),"No table found in this data source.",Global.tabName,JOptionPane.INFORMATION_MESSAGE);
				else {
					setConnectedGUI(true);
					connected = false;
					sysCheckBox.setEnabled(false);
					tableCheckBox.setEnabled(false);
					viewCheckBox.setEnabled(false);
					otherCheckBox.setEnabled(false);
					for (i = 0; i < tableTypes.length; i++) {
						if (tableTypes[i].compareToIgnoreCase("SYSTEM TABLE") == 0) {
							sysCheckBox.setEnabled(true);
							sysCheckBox.setSelected(true);
						} else if (tableTypes[i].compareToIgnoreCase("TABLE") == 0) {
							tableCheckBox.setEnabled(true);
							tableCheckBox.setSelected(true);
						} else if (tableTypes[i].compareToIgnoreCase("VIEW") == 0) {
							viewCheckBox.setEnabled(true);
							viewCheckBox.setSelected(true);
						} else {
							otherCheckBox.setEnabled(true);
							otherCheckBox.setSelected(true);
						}
					}

					connected = true;
				}
            } catch (SQLException ex) {
		        cmbDSN.removeItem(cmbDSN.getSelectedItem());
				Global.defaultSQLExceptionHandler(JOptionPane.getFrameForComponent(this),ex);
			} catch (ClassNotFoundException cnfe) {
				Global.defaultClassNotFoundExceptionHandler(JOptionPane.getFrameForComponent(this), cnfe);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
		} else if (source == infoButton) {
			DataSourceInfoDialog dlg = new DataSourceInfoDialog(
				JOptionPane.getFrameForComponent(this),connectInfo);
			dlg.setLocationRelativeTo(JOptionPane.getFrameForComponent(this));
			dlg.showDialog();
        } else if (source == btnDumpToProtegeProject) {
			if (nImportMethod == IMPORT_HEURISTIC) {
				//if (heuristicDlg == null)
				heuristicDlg = new HeuristicImportDlg(JOptionPane.getFrameForComponent(this),tables,connectInfo,kb);
				//heuristicDlg.setNewData(tables,connectInfo);
				heuristicDlg.setLocationRelativeTo(JOptionPane.getFrameForComponent(this));
				if (heuristicDlg.showDialog()) {
					dump();
					heuristicDlg.setUpInstancePtrsMtoM();
				}
			} else
				dump();
        }
        
        String strActionCmd = e.getActionCommand();
		if (strActionCmd == "optEntireDB") {
			selectAllDBTables();
			nImportMethod = IMPORT_ENTIRE_DB;
		} else if (strActionCmd == "optSelected") {
			nImportMethod = IMPORT_SELECTED;
		} else if (strActionCmd == "optHeuristic") {
			selectAllDBTables();
			nImportMethod = IMPORT_HEURISTIC;
		} else if (strActionCmd == "optODBC") {
			nDSType = ODBC;
			driverLabel.setText("ODBC Driver");
			dsLabel.setText("Data Source Name jdbc:odbc:");
			txtDriver.setText(strODBCDriver);
			txtDriver.setEditable(false);
		} else if (strActionCmd == "optJDBC") {
			nDSType = JDBC;
			driverLabel.setText("JDBC Driver");
			dsLabel.setText("JDBC URL ");
			txtDriver.setEditable(true);
		}
    }

	// Handle the state changes in the table type check boxes;
	// modify the table tree to show the appropriate tables

	public void itemStateChanged(ItemEvent e) {
		if (connected) {
			Object item = e.getItem();
			Vector selectTypes = new Vector();
			if (sysCheckBox.isSelected())
				selectTypes.addElement("SYSTEM TABLE");
			if (tableCheckBox.isSelected())
				selectTypes.addElement("TABLE");
			if (viewCheckBox.isSelected())
				selectTypes.addElement("VIEW");
			if (otherCheckBox.isSelected()) {
				selectTypes.addElement("GLOBAL TEMPORARY");
				selectTypes.addElement("LOCAL TEMPORARY");
				selectTypes.addElement("ALIAS");
				selectTypes.addElement("SYNONYM");
			}
			
			int numTypes = selectTypes.size();
			String[] selectTableTypes = new String[numTypes];
			for (int i = 0; i < numTypes; i++)
				selectTableTypes[i] = (String)selectTypes.get(i);

			selectTableTypesForView(selectTableTypes);
		}
	}

	// Take a list of table types and modify the tree to show all the
	// tables of those types.
	
	private void selectTableTypesForView(String[] tableTypes) {
		// reset data and GUI components
        previewPanel.deleteTable();
		tables.removeAllElements();			
        rootNode = new DefaultMutableTreeNode(new TreeNodeInfo("Data Source",TreeNodeInfo.OTHER));
        treeModel = new DefaultTreeModel(rootNode);
        treeTables.setModel(treeModel);
		nImportMethod = IMPORT_SELECTED;
		optSelected.doClick();
		//setConnectedGUI(false);

        try {
	       	String strURL = "";
			Connection con = null;
			Class.forName(txtDriver.getText());
			if(nDSType == ODBC)
				strURL = "jdbc:odbc:" + (String)cmbDSN.getSelectedItem(); 
			else 
				strURL = (String)cmbDSN.getSelectedItem(); 
				
			con = DriverManager.getConnection(strURL, txtUserID.getText(), new String(txtPW.getPassword()));
                
            DatabaseMetaData dbMetaData = con.getMetaData();

			ResultSet rsTables = dbMetaData.getTables(null,null,null,tableTypes);
		    boolean bMore = rsTables.next();
		    while (bMore) {
				String strTableName = rsTables.getString("TABLE_NAME");

				// Determine the table type
				String tableTypeStr = rsTables.getString("TABLE_TYPE");
				int tableType;
				if (tableTypeStr.compareToIgnoreCase("SYSTEM TABLE") == 0)
					tableType = TreeNodeInfo.TABLE_SYSTEM;
				else if (tableTypeStr.compareToIgnoreCase("TABLE") == 0)
					tableType = TreeNodeInfo.TABLE;
				else if (tableTypeStr.compareToIgnoreCase("VIEW") == 0)
					tableType = TreeNodeInfo.TABLE_VIEW;
				else
					tableType = TreeNodeInfo.TABLE_OTHER;

				// Get primary keys
				Vector primaryKeys = new Vector();

				try {
					ResultSet rs = dbMetaData.getPrimaryKeys(null,null,strTableName);
					boolean more = rs.next();
					while (more) {
						primaryKeys.add(rs.getString("COLUMN_NAME"));
						more = rs.next();
					}
					rs.close();
				} catch (Exception e) { System.out.println("QUERYING A TABLE FOR PRIMARY KEYS IS NOT SUPPORTED BY THIS DRIVER"); }

    			    
		        TreeNodeInfo nodeInfo = new TreeNodeInfo(strTableName,tableType);
				DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(nodeInfo);
			    treeModel.insertNodeInto(childNode, rootNode, rootNode.getChildCount());
			    treeTables.scrollPathToVisible(new TreePath(childNode.getPath()));
			    ResultSet rsColumns = dbMetaData.getColumns(null,null,strTableName,null);
			    boolean bMore2 = rsColumns.next();
			    while (bMore2) {
			        String strColumnName = rsColumns.getString("COLUMN_NAME");
			        String strTypeName = rsColumns.getString("TYPE_NAME");
					int sqlType = rsColumns.getShort("DATA_TYPE");
					ValueType protegeType = Global.getSQLProtegeType(sqlType);
					String nodeText = strColumnName+" ("+strTypeName+" --> "+protegeType.toString()+")";

					// Determine whether this column is a primary key
					int colType = TreeNodeInfo.COLUMN;
					for (int i = 0; i < primaryKeys.size(); i++) {
						String key = (String)primaryKeys.elementAt(i);
						if (key.compareToIgnoreCase(strColumnName) == 0) {
							colType = TreeNodeInfo.COLUMN_PK;
							break;
						}
					}

			        TreeNodeInfo nodeInfo2 = new TreeNodeInfo(nodeText,colType);
					DefaultMutableTreeNode childColumnNode = new DefaultMutableTreeNode(nodeInfo2);
			        treeModel.insertNodeInto(childColumnNode, childNode, childNode.getChildCount());
    		    
			        bMore2 = rsColumns.next();
			    }
			    rsColumns.close();
    			
			    tables.add(nodeInfo);
				
				bMore = rsTables.next();
		    }
    		
		    rsTables.close();
		    con.close();
		    
			if (tables.size() == 0) {
				JOptionPane.showMessageDialog(JOptionPane.getFrameForComponent(this),"No table of the selected types above found.",Global.tabName,JOptionPane.INFORMATION_MESSAGE);
			}
				/*
			else
				setConnectedGUI(true);
				*/
        } catch (SQLException ex) {
			Global.defaultSQLExceptionHandler(JOptionPane.getFrameForComponent(this),ex);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
	}
    
	// This method is used to enable/disable the GUI components based
	// on the current state of the tab widget

	private void setConnectedGUI(boolean isConnected) {
		connected = isConnected;

//		allCheckBox.setEnabled(isConnected);
		sysCheckBox.setEnabled(isConnected);
		tableCheckBox.setEnabled(isConnected);
		viewCheckBox.setEnabled(isConnected);
		otherCheckBox.setEnabled(isConnected);

		if (!isConnected) {
//			allCheckBox.setSelected(isConnected);
			sysCheckBox.setSelected(isConnected);
			tableCheckBox.setSelected(isConnected);
			viewCheckBox.setSelected(isConnected);
			otherCheckBox.setSelected(isConnected);
		}

		infoButton.setEnabled(isConnected);

		optEntireDB.setEnabled(isConnected);
		optSelected.setEnabled(isConnected);
		optHeuristic.setEnabled(isConnected);

		btnDumpToProtegeProject.setEnabled(isConnected);
	}

	// Select all the tables in the tree.  These are the top-level
	// items in the tree.
	
	private void selectAllDBTables() {
		TreePath paths[] = new TreePath[rootNode.getChildCount()];
		Enumeration enum = rootNode.children();
		int i = 0;
		while (enum.hasMoreElements()) {
			TreeNode path[] = treeModel.getPathToRoot((TreeNode)enum.nextElement());
			paths[i] = new TreePath(path);
			i++;
		}
		treeTables.setSelectionPaths(paths);
	}

	// Import the selected tables from the data source into the Protege project
	
	private void dump() {
		TreePath[] selection = treeTables.getSelectionPaths();
		if (selection != null) {
			String strURL = "";
        
			try {
				Connection con = null;
				Class.forName(txtDriver.getText());
				if(nDSType == ODBC)
					strURL = "jdbc:odbc:" + (String)cmbDSN.getSelectedItem(); 
				else 
					strURL = (String)cmbDSN.getSelectedItem(); 
				
				con = DriverManager.getConnection(strURL, txtUserID.getText(), new String(txtPW.getPassword()));
                
				DatabaseMetaData dbMetaData = con.getMetaData();

				// First, create a Table Metaclass to hold meta-information about each table (primary key, foreign keys, bridge table, etc.)
				prepareKB();

				for (int i = 0; i < selection.length; i++) {
					// Get the table to insert into the Protege project
					DefaultMutableTreeNode node = (DefaultMutableTreeNode)selection[i].getLastPathComponent();
					TreeNodeInfo nodeInfo = (TreeNodeInfo)node.getUserObject();
					String strTableName = nodeInfo.m_text;

					// Create the Protege class and slots for each table
					Global.debug("Importing table '" + strTableName+"' to class '" + strTableName+"'.");

					Cls cls = kb.createCls(strTableName, kb.getRootClses(), this.metaClass);

					// Set the primary key fields
					Slot pks = kb.getSlot("Primary Key Fields");
					cls.setOwnSlotValues(pks, nodeInfo.primaryKeyFields);
					
					// Check for bridge table
					if(nodeInfo.primaryKeyFields.size() == 2 && nodeInfo.foreignKeys.size() == 2)
					{
						String field1 = ((ForeignKey)nodeInfo.foreignKeys.get(0)).localField;
						String field2 = ((ForeignKey)nodeInfo.foreignKeys.get(1)).localField;
						if(contains(nodeInfo.primaryKeyFields, field1) &&
						   contains(nodeInfo.primaryKeyFields, field2))
							cls.setOwnSlotValue(this.kb.getSlot("Is Bridge Table"), new Boolean(true));
					}
					else
						cls.setOwnSlotValue(this.kb.getSlot("Is Bridge Table"), new Boolean(false));
					
					// Set the foreign key instances
					Cls fkCls = this.kb.getCls("Foreign Key");
					Slot fks = this.kb.getSlot("Foreign Keys");
					ArrayList fkInstances = new ArrayList();
					
					for(int f = 0; f < nodeInfo.foreignKeys.size(); f++)
					{
						ForeignKey fk = (ForeignKey)nodeInfo.foreignKeys.get(f);
						// Create an instance of the foreign key
						String insName = "Foreign_Key_Instance_" + this.instanceCount++;
						Instance ins = this.kb.createInstance(insName, fkCls);
						ins.addOwnSlotValue(this.kb.getSlot("FK Name"), fk.name);
						ins.addOwnSlotValue(this.kb.getSlot("Local Field"), fk.localField);
						ins.addOwnSlotValue(this.kb.getSlot("Reference Field"), fk.referenceField);
						ins.addOwnSlotValue(this.kb.getSlot("Reference Table"), fk.referenceTable);
						
						// Add the instance to a list
						fkInstances.add(ins);	
					}
					cls.setOwnSlotValues(fks, fkInstances);

					ResultSet rsColumns = dbMetaData.getColumns(null,null,strTableName,null);
					boolean bMore = rsColumns.next();
					while (bMore) {
						String strColumnName = rsColumns.getString("COLUMN_NAME");
						int sqlType = rsColumns.getShort("DATA_TYPE");
						ValueType protegeType = Global.getSQLProtegeType(sqlType);

						Slot slot = kb.getSlot(strColumnName);
						if (slot == null) {
							Slot newSlot = kb.createSlot(strColumnName);
							newSlot.setValueType(protegeType);
							cls.addDirectTemplateSlot(newSlot);
						} else {							
							if (slot.getValueType() == protegeType)
								cls.addDirectTemplateSlot(slot);
							else {								
								java.awt.Frame frame = JOptionPane.getFrameForComponent(this);
								NewSlotNameDialog dialog = new NewSlotNameDialog(frame,strColumnName);
								dialog.setLocationRelativeTo(JOptionPane.getFrameForComponent(this));
								while (true) {
									if (dialog.showDialog()) {
										slot = kb.getSlot(dialog.selectedSlotName);
										if (slot == null)
											break;
									}
								}
								Slot newSlot = kb.createSlot(dialog.selectedSlotName);
								newSlot.setValueType(protegeType);
								cls.addDirectTemplateSlot(newSlot);
							}
						}

						bMore = rsColumns.next();
					}
					rsColumns.close();
            
					// Create class instances from the rows of data in the table
					strTableName = getDBMSSpecificNamingStyle(strTableName);
					
					String strQuery = "SELECT * FROM " + strTableName;

					Statement stmt = con.createStatement();
					ResultSet resultSet = stmt.executeQuery(strQuery);
					ResultSetMetaData rsmd = resultSet.getMetaData();

					int i2 = 0;
					while (resultSet.next()) {
						String strInsName = cls.getName()+"_Instance_"+Integer.toString(i2+1);
						Instance ins = kb.createInstance(strInsName, cls);
						
						for(int j = 0; j < rsmd.getColumnCount(); j++)
						{
							Object obj = resultSet.getObject(j+1);
							int sqlType = rsmd.getColumnType(j+1);
							String colName = rsmd.getColumnName(j+1);
							obj = Global.getSQLProtegeObject(sqlType, obj);
							if (obj != null)						
								ins.addOwnSlotValue(this.kb.getSlot(colName), obj);
						}
						
						i2++;
					}

					resultSet.close();
					stmt.close();
					
					Global.debug("Import Successful.\n" + i2 + " instance(s) imported.");
				}
            
            	// Next, do some refactoring of the data, to try to handle foreign key relationships and bridge tables
            	refactor();
            	
				// Let the user know that the import is sucessful.
				kb.setChanged(true);
				con.close();
				String strMsg = "Import Completed";
				JOptionPane.showMessageDialog(JOptionPane.getFrameForComponent(this),strMsg,Global.tabName,JOptionPane.INFORMATION_MESSAGE);
			} catch (SQLException ex) {
				Global.defaultSQLExceptionHandler(JOptionPane.getFrameForComponent(this),ex);
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		} else {
			String msg = "Please select the tables to import.\nNo table is selected.";
			JOptionPane.showMessageDialog(JOptionPane.getFrameForComponent(this),msg,Global.tabName,JOptionPane.INFORMATION_MESSAGE);
		}
	}

	private void prepareKB()
	{    	
    	// Create the metaclass for all tables
    	this.metaClass = this.kb.getCls("Table Metaclass");
    	Collection tempholder = new ArrayList();
    	if(this.metaClass == null)
    	{
	    	tempholder = new Vector();
	    	tempholder.add(this.kb.getCls(":STANDARD-CLASS"));
	    	this.metaClass = this.kb.createCls("Table Metaclass", tempholder);
	    }

    	// Create the class Foreign Key
    	tempholder = new ArrayList();
    	tempholder.add(this.kb.getCls(":SYSTEM-CLASS"));
    	Cls fkClass = this.kb.getCls("Foreign Key");
    	Cls bridgeClass = this.kb.getCls("Bridge Table");
    	if(fkClass == null)
    	{
    		fkClass = this.kb.createCls("Foreign Key", tempholder);
    	
	    	// Add a slot for name
		    Slot fkname = this.kb.getSlot("FK Name");
		    if(fkname == null)
		    {
		   		fkname = this.kb.createSlot("FK Name");
		    	fkname.setValueType(ValueType.STRING);
		    	fkname.setMaximumCardinality(1);
		    }
	    	fkClass.addDirectTemplateSlot(fkname);
	    	
	    	// Add a slot for local field
		    Slot local = this.kb.getSlot("Local Field");
		    if(local == null)
		    {
		   		local = this.kb.createSlot("Local Field");
		    	local.setValueType(ValueType.STRING);
		    	local.setMaximumCardinality(1);
		    }
	    	fkClass.addDirectTemplateSlot(local);
	    	
	    	// Add a slot for reference field
		    Slot reference = this.kb.getSlot("Reference Field");
		    if(reference == null)
		    {
		   		reference = this.kb.createSlot("Reference Field");
		    	reference.setValueType(ValueType.STRING);
		    	reference.setMaximumCardinality(1);
		    }
	    	fkClass.addDirectTemplateSlot(reference);
	    	
	    	// Add a slot for reference table
		    Slot refTable = this.kb.getSlot("Reference Table");
		    if(refTable == null)
		    {
		   		refTable = this.kb.createSlot("Reference Table");
		    	refTable.setValueType(ValueType.STRING);
		    	refTable.setMaximumCardinality(1);
		    }
	    	fkClass.addDirectTemplateSlot(refTable);
	    		    	
	    	// Add primary key field(s) to the metaclass
		    Slot pks = this.kb.getSlot("Primary Key Fields");
		    if(pks == null)
		    {
		   		pks = this.kb.createSlot("Primary Key Fields");
		    	pks.setValueType(ValueType.STRING);
		    	pks.setAllowsMultipleValues(true);
		    }
	    	metaClass.addDirectTemplateSlot(pks);
	    	
	    	// Add foreign key relationships to the metaclass
		    Slot fks = this.kb.getSlot("Foreign Keys");
		    if(fks == null)
		    {
		   		fks = this.kb.createSlot("Foreign Keys");
		    	fks.setValueType(ValueType.INSTANCE);
		    	Collection classholder = new ArrayList();
		    	classholder.add(this.kb.getCls("Foreign Key"));
		    	fks.setAllowedClses(classholder);
		    	fks.setAllowsMultipleValues(true);
		    }
	    	metaClass.addDirectTemplateSlot(fks);

	    	// Add a checkbox to the metaclass to denote whether this might be a bridge table
		    Slot bridge = this.kb.getSlot("Is Bridge Table");
		    if(bridge == null)
		    {
		   		bridge = this.kb.createSlot("Is Bridge Table");
		    	bridge.setValueType(ValueType.BOOLEAN);
		    }
	
	    	metaClass.addDirectTemplateSlot(bridge);
	    }		
	}

	// Iterate through the class ontology. If a class has values in its "foreign keys" slot (but is not deemed 
	// a bridge table), prepare to do some manipulation. 
	//		(i) First, for each foreign key reference, add a new slot, FKINST_i, to it whose value type 
	//			is instances of the foreign key class (the reason we could not do this before is because 
	//			the foreign key class might not have been created yet).
	//		(ii) Next, iterate through the instances of this class. For each instance, fill in the instances 
	//			of FKINST by selecting instances of the foreign key class whose PK matches the FK you have 
	//			stored in each instance of this class.
	//		(iii) Lastly, remove this class from the domain for the FK slot, since you will no longer need that 
	//			integer value. This is for cleanup purposes.	
	private void refactor()
	{
		Iterator clsesIter = this.kb.getClses().iterator();
		Slot pkSlot = this.kb.getSlot("Primary Key Fields");
		Slot fkSlot = this.kb.getSlot("Foreign Keys");
		Slot bridgeSlot = this.kb.getSlot("Is Bridge Table");
		Slot fkLocalFieldSlot = this.kb.getSlot("Local Field");
		Slot fkReferenceField = this.kb.getSlot("Reference Field");
		Slot fkReferenceTable = this.kb.getSlot("Reference Table");
		
		while(clsesIter.hasNext())
		{
        	try {
				Cls cls = (Cls) clsesIter.next();
				String clsName = cls.getName();
				if(cls.getDirectType().getName().equals("Table Metaclass"))
				{
					if (((Boolean)cls.getOwnSlotValue(bridgeSlot)).booleanValue()) // It is a bridge table (many-to-many)
					{
						// Look at the foreign keys to determine the bridge structure
						Collection fks = cls.getOwnSlotValues(fkSlot);  // These will be instances of Foreign Key
						Iterator fkIter = fks.iterator();
						Instance fk1 = (Instance)fkIter.next();
						Instance fk2 = (Instance)fkIter.next();
						String tableName1 = (String)fk1.getOwnSlotValue(fkReferenceTable);  // One side of the bridge...
						String tableName2 = (String)fk2.getOwnSlotValue(fkReferenceTable);	// ...and the other
						String localField1 = (String) fk1.getOwnSlotValue(fkLocalFieldSlot);
						String localField2 = (String) fk2.getOwnSlotValue(fkLocalFieldSlot);
						String referenceField1 = (String) fk1.getOwnSlotValue(fkReferenceField);
						String referenceField2 = (String) fk2.getOwnSlotValue(fkReferenceField);
						
						// Create new slots in each of the bridged tables, and make them inverses of each other
						Slot newSlot1 = generateSlot(this.kb.getCls(tableName1), tableName2 + " Instances", this.kb.getCls(tableName2), true);
						Slot newSlot2 = generateSlot(this.kb.getCls(tableName2), tableName1 + " Instances", this.kb.getCls(tableName1), true);
						newSlot1.setInverseSlot(newSlot2);						
						
						// Iterate through the instances of this class, and assign values to the slots that were just created
						Collection instances = cls.getDirectInstances();
						Collection referenceInstances1 = this.kb.getCls(tableName1).getDirectInstances();
						Collection referenceInstances2 = this.kb.getCls(tableName2).getDirectInstances();
						Iterator instIter = instances.iterator();
						while(instIter.hasNext())
						{
							Instance localInstance = (Instance)instIter.next();
							int idToMatch1 = ((Integer)localInstance.getOwnSlotValue(this.kb.getSlot(localField1))).intValue();  // value to match in table1	
							int idToMatch2 = ((Integer)localInstance.getOwnSlotValue(this.kb.getSlot(localField2))).intValue();  // value to match in table2

							Instance referenceInstance1 = getMatchingInstance(referenceInstances1, referenceField1, idToMatch1);
							Instance referenceInstance2 = getMatchingInstance(referenceInstances2, referenceField2, idToMatch2);
							if(referenceInstance1 == null)
								System.out.println("ERROR: Could not find matching instance for " + referenceField1 + " = " + idToMatch1);
							else if(referenceInstance2 == null)
								System.out.println("ERROR: Could not find matching instance for " + referenceField2 + " = " + idToMatch2);
							else
								referenceInstance1.addOwnSlotValue(newSlot1, referenceInstance2);	
						}
					
					}
					else // Treat foreign keys as a many-to-one relationship
					{
						// Process the list of foreign keys, making the appropriate substitutions
						Collection pks = cls.getOwnSlotValues(pkSlot);  // These will be Strings
						Collection fks = cls.getOwnSlotValues(fkSlot);  // These will be instances of Foreign Key
						Collection instances = cls.getDirectInstances();
						
						// Iterate through the foreign keys, checking to see if they are part of the primary key
						Iterator iter = fks.iterator();
						while(iter.hasNext())
						{
							Instance fkInst = (Instance)iter.next();
							String localField = (String) fkInst.getOwnSlotValue(fkLocalFieldSlot);
							String referenceField = (String) fkInst.getOwnSlotValue(fkReferenceField);
							String referenceTable = (String) fkInst.getOwnSlotValue(fkReferenceTable);
						
							// If it is not part of the PK, create a new slot to hold instances of the foreign table
							if(!contains(pks, localField))
							{
								Cls refTableCls = this.kb.getCls(referenceTable);
								if(refTableCls != null)
								{
									Slot newSlot = generateSlot(cls, localField + "_INSTANCE", refTableCls, false);
								
									// Iterate through the instances of this class, and for each one, fill in the new slot with
									// instances of the foreign table.
									Iterator localInstanceIterator = instances.iterator();
									Collection referenceInstances = this.kb.getCls(referenceTable).getDirectInstances();
									while(localInstanceIterator.hasNext())
									{
										Instance localInstance = (Instance) localInstanceIterator.next();
										int idToMatch = ((Integer)localInstance.getOwnSlotValue(this.kb.getSlot(localField))).intValue();
										Instance referenceInstance = getMatchingInstance(referenceInstances, referenceField, idToMatch);
										if(referenceInstance == null)
											System.out.println("ERROR: Could not find matching instance for " + referenceField + " = " + idToMatch);
										else
											localInstance.setOwnSlotValue(newSlot, referenceInstance);	
									}
								}
							}	
						}
					}
				}
			} catch (Exception e) { e.printStackTrace(); }
		}		
	}

	// This method creates a new slot in "thisCls" named "name", whose value is of type "allowedCls"
	public Slot generateSlot(Cls thisCls, String name, Cls allowedCls, boolean allowMult)
	{
		Slot newSlot = this.kb.getSlot(name);
		if(newSlot == null)
			newSlot= this.kb.createSlot(name);
		newSlot.setValueType(ValueType.INSTANCE);
		ArrayList classHolder = new ArrayList();
		classHolder.add(allowedCls);
		newSlot.setAllowedClses(classHolder);
		newSlot.setAllowsMultipleValues(allowMult);
		thisCls.addDirectTemplateSlot(newSlot);
		
		return newSlot;
	}
	
	// Returns true if String s is in collection c.
	private boolean contains(Collection c, String s)
	{
		Iterator iter = c.iterator();
		while(iter.hasNext())
		{
			if(iter.next().toString().equals(s))
				return true;	
		}
		
		return false;
	}
	
	// This method scans a list of instances for one where the value of slotName matches id.
	private Instance getMatchingInstance(Collection instances, String slotName, int id)
	{
		Slot s = this.kb.getSlot(slotName);
		Iterator iter = instances.iterator();
		while(iter.hasNext())
		{
			Instance inst = (Instance)iter.next();
			int val = ((Integer)inst.getOwnSlotValue(s)).intValue();
			if(val == id)
				return inst;
		}
		
		return null;
	}

    // A new table has been selected from the tree and we want to display
	// a preview of it in the preview table

	public void valueChanged(TreeSelectionEvent e) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)treeTables.getLastSelectedPathComponent();
        if (node == null)
            return;
        
        TreeNodeInfo nodeInfo = (TreeNodeInfo)node.getUserObject();
        String strTableName = getDBMSSpecificNamingStyle(nodeInfo.m_text);
		
		String strQuery = "SELECT * FROM " + strTableName;
        previewPanel.updateTable(connectInfo,strQuery);
    }
    
	// Return the list of data source names
	
	public Vector getDSNList() {
		Vector dsnList = new Vector();
		int count = cmbDSN.getItemCount();
		for (int i = 0; i < count; i++)
			dsnList.addElement(cmbDSN.getItemAt(i));
		
		return dsnList;		
	}

	// Different DBMS handle table names in queries differently.
	// For instance, MySQL does not allow characters such as spaces,
	// $, #, ', and other in table names while they are valid in DBMS such as
	// MS Access and Excel.  Therefore, this method is intended to modify the 
	// table names accordingly to work with these different DBMS.

	private String getDBMSSpecificNamingStyle(String tableName) {
		if (tableName.indexOf(' ') != -1
			|| tableName.indexOf('$') != -1 
			|| tableName.indexOf('#') != -1
			|| tableName.indexOf('\'') != -1)
			return "\"" + tableName + "\"";
		else
			return tableName;
	}


	// Class data

    private static final int IMPORT_ENTIRE_DB = 1;
	private static final int IMPORT_SELECTED = 2;
	private static final int IMPORT_HEURISTIC = 3;

	private static int nImportMethod = IMPORT_SELECTED;

	private static final int ODBC = 1;
	private static final int JDBC = 2;
	
	private static int nDSType = ODBC;
	
	private JRadioButton optJDBC;
	private JRadioButton optODBC;

	private JRadioButton optEntireDB;
	private JRadioButton optSelected;
	private JRadioButton optHeuristic;

	private HeuristicImportDlg heuristicDlg;

	private JCheckBox sysCheckBox, tableCheckBox, viewCheckBox, otherCheckBox;
	private String[] tableTypes;

	private Vector tables;
	
	private String strODBCDriver = "sun.jdbc.odbc.JdbcOdbcDriver";

	private JLabel driverLabel = new JLabel("ODBC Driver ", JLabel.LEFT);
	private JLabel dsLabel = new JLabel("Data Source Name ", JLabel.LEFT);
    private JComboBox cmbDSN;
    private JTextField txtDriver = new JTextField(20);
    private JTextField txtUserID = new JTextField(20);
    private JPasswordField txtPW = new JPasswordField(20);
    private JButton btnConnect = new JButton("Connect");
	private JButton infoButton = new JButton("Info");

	private boolean connected = false;

	private ConnectInfo connectInfo = null;
    
    private DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode(new TreeNodeInfo("Data Source",TreeNodeInfo.OTHER));
    private DefaultTreeModel treeModel = new DefaultTreeModel(rootNode);
    private JTree treeTables = new JTree(treeModel);
    
    private JButton btnDumpToProtegeProject = new JButton("Begin Importing");
    
    private DataPreviewPanel previewPanel;
    private KnowledgeBase kb;
    private Cls metaClass;

	private DataSourceInfo dsInfo;
	
	private int instanceCount = 1;
}

class TreeNodeInfo {

	public TreeNodeInfo(String text, int type) {
		m_text = text;
		m_type = type;
	}
	
	public TreeNodeInfo(String text, int type, int sqlType) {
		m_text = text;
		m_type = type;
		m_colSQLType = sqlType;
	}

	public String toString() {
		return m_text;
	}
	
	public String m_text;
	public int m_type;
	public int m_colSQLType;
	public ArrayList primaryKeyFields = new ArrayList();
	public ArrayList foreignKeys = new ArrayList();

	public static final int TABLE_SYSTEM = 1;
	public static final int TABLE = 2;
	public static final int TABLE_VIEW = 3;
	public static final int TABLE_OTHER = 10;

	public static final int COLUMN = 100;
	public static final int COLUMN_PK = 101;	

	public static final int OTHER = -1;
}

class ForeignKey {
	public ForeignKey(String n, String l, String f, String t)
	{
		name = n;
		localField = l;
		referenceField = f;
		referenceTable = t;
	}
	
	public String toString()
	{
		StringBuffer buf = new StringBuffer();
		buf.append("Name: " + name);
		buf.append("\nLocal Field: " + localField);
		buf.append("\nRef Field: " + referenceField);
		buf.append("\nRef Table: " + referenceTable);
		return buf.toString();
	}
	
	public String name;
	public String localField;
	public String referenceField;
	public String referenceTable;	
}

class DataSourceInfo {

	public DataSourceInfo(ConnectInfo connectInfo) {
		m_connectInfo = connectInfo;

		try {
			Class.forName(m_connectInfo.m_driver);

			Connection con = DriverManager.getConnection(m_connectInfo.m_DSN, m_connectInfo.m_UID, m_connectInfo.m_PWD);

			DatabaseMetaData dbmd = con.getMetaData();

			m_supportsANSI92EntryLevelSQL = dbmd.supportsANSI92EntryLevelSQL();
			m_supportsANSI92FullSQL = dbmd.supportsANSI92FullSQL();
			m_supportsANSI92IntermediateSQL = dbmd.supportsANSI92IntermediateSQL();
			m_supportsCoreSQLGrammar = dbmd.supportsCoreSQLGrammar();

			con.close();
		} catch (SQLException e) {
			//Global.defaultSQLExceptionHandler(JOptionPane.getFrameForComponent(this),e);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
	}

	public boolean m_supportsANSI92EntryLevelSQL;
	public boolean m_supportsANSI92FullSQL;
	public boolean m_supportsANSI92IntermediateSQL;
	public boolean m_supportsCoreSQLGrammar;

	private ConnectInfo m_connectInfo;
}

class MyListCellRenderer extends DefaultListCellRenderer {

	// Constructor

	public MyListCellRenderer() {
		sysTableIcon = new ImageIcon("resource/table_system.gif");
		tableIcon = new ImageIcon("resource/table.gif");
		viewIcon = new ImageIcon("resource/view.gif");
		middleIcon = new ImageIcon("resource/middle.gif");

		pkIcon = new ImageIcon("resource/pk.gif");
	}

	// Override
	
	public Component getListCellRendererComponent(JList list,
												  Object value,
												  int index,
												  boolean isSelected,
												  boolean cellHasFocus) {
		super.getListCellRendererComponent(list,value,index,isSelected,cellHasFocus);

		TreeNodeInfo info = (TreeNodeInfo)value;

		Font font = getFont();
		switch (info.m_type) {
			case TreeNodeInfo.TABLE_SYSTEM:
				setFont(font.deriveFont(Font.ITALIC));
				setIcon(sysTableIcon);
				break;
			case TreeNodeInfo.TABLE:
				setFont(font.deriveFont(Font.BOLD));
				setIcon(tableIcon);
				break;
			case TreeNodeInfo.TABLE_VIEW:
				setFont(font.deriveFont(Font.PLAIN));
				setIcon(viewIcon);
				break;
			case TreeNodeInfo.COLUMN:
				setFont(font.deriveFont(Font.PLAIN));
				break;
			case TreeNodeInfo.COLUMN_PK:
				setFont(font.deriveFont(Font.BOLD));
				setIcon(pkIcon);
				break;
		}

        return this;
	}
	
	ImageIcon sysTableIcon;
	ImageIcon tableIcon;
	ImageIcon viewIcon;
	ImageIcon middleIcon;

	ImageIcon pkIcon;
}

