/* ////////////////////////////////////////////////////////////////////////////

// Project: Sharing data from a RDBMS with a Protege project using JDBC/ODBC.

// Developer: My Nguyen
// Developer: Adam Silberfein (2005 updates)

/////////////////////////////////////////////////////////////////////////////*/

package DataGenieTab;

///////////////////////////////////////////////////////////////////////////////
// IMPORTS

// For the JDBC
import java.sql.*;

// For the Protege PDK
import edu.stanford.smi.protege.model.*;

// For the Swing UI Components
import javax.swing.*;

// For other Java SDK packages
import java.math.*;


///////////////////////////////////////////////////////////////////////////////
// Global

// The Global class contains common services used by other objects

class Global extends Object {

	// Information about the tab and the current version

	static public final String tabName = "DataGenie";
	static public final String tabVersion = "v2.0b";

	// return a Protege equivalent value type of a SQL data type.
	// This method also safeguards against triggering the new strong
	// type checking in the current version 1.6 build 787 of Protege

	static public ValueType getSQLProtegeType(int sqlType) {	
        switch(sqlType) {
        case Types.BIT:
			return ValueType.BOOLEAN;
			// return ValueType.CLS;
        case Types.FLOAT:
        case Types.DOUBLE:
		case Types.REAL:
		case Types.NUMERIC:
		case Types.DECIMAL:
			return ValueType.FLOAT;
			// return ValueType.INSTANCE;
        case Types.TINYINT:
        case Types.SMALLINT:
        case Types.INTEGER:
        case Types.BIGINT:
			return ValueType.INTEGER;
        case Types.CHAR:
        case Types.VARCHAR:
        case Types.LONGVARCHAR:
        case Types.DATE:
        case Types.TIME:
        case Types.TIMESTAMP:
			return ValueType.STRING;
			// return ValueType.SYMBOL;
		case Types.ARRAY:
		case Types.BINARY:
		case Types.BLOB:
		case Types.CLOB:
		// Types.DISTINT:
		case Types.JAVA_OBJECT:
		case Types.LONGVARBINARY:
		case Types.NULL:
		case Types.OTHER:
		case Types.REF:
		case Types.STRUCT:
		case Types.VARBINARY:
        default:		
			return ValueType.ANY;
        }
	}

	// Similar to the above method, but converts a SQL object to
	// a Protege object.

	// See "8. Mapping SQL data types into Java, Table 2: Standard mapping
	// from SQL types to Java types" (D:\Program Files\jdk1.3\docs\guide\jdbc\spec\jdbc-spec.frame8.html)
	// for the standard mapping from SQL tyes to Java types.

	static public Object getSQLProtegeObject(int sqlType, Object obj) {	
		if (obj == null)
			return obj;

        switch(sqlType) {
        case Types.BIT:
			return (Boolean)obj;

        case Types.FLOAT:
			return new Float(((Double)obj).floatValue());
        case Types.DOUBLE:
            // because Protege only has type FLOAT for floating point number
            // and not a type DOUBLE for double (Double.class), we need to convert to Float
			// will get an exception otherwise
			return new Float(((Double)obj).floatValue());
		case Types.REAL:
			return (Float)obj;
		case Types.NUMERIC:
			return new Float(((BigDecimal)obj).floatValue());
		case Types.DECIMAL:
			return new Float(((BigDecimal)obj).floatValue());

        case Types.TINYINT:
			//return new Integer(((Byte)obj).intValue());
			return (Integer)obj;
        case Types.SMALLINT:
			//return new Integer(((Short)obj).intValue());
			return (Integer)obj;
        case Types.INTEGER:
			return (Integer)obj;
        case Types.BIGINT:
			//return new Integer(((BigInteger)obj).intValue());
			return new Integer(((Long)obj).intValue());

        case Types.CHAR:
			return (String)obj;
        case Types.VARCHAR:
			return (String)obj;
        case Types.LONGVARCHAR:
			return (String)obj;
        case Types.DATE:
			return obj.toString();
        case Types.TIME:
			return obj.toString();
        case Types.TIMESTAMP:
			return obj.toString();

		case Types.ARRAY:
		case Types.BINARY:
		case Types.BLOB:
		case Types.CLOB:
		// Types.DISTINT:
		case Types.JAVA_OBJECT:
		case Types.LONGVARBINARY:
		case Types.NULL:
		case Types.OTHER:
		case Types.REF:
		case Types.STRUCT:
		case Types.VARBINARY:
        default:		
			return obj;
        }
	}

	// The default handler for SQL exceptions in this project

	static public void defaultSQLExceptionHandler(java.awt.Frame parent, SQLException ex) {
		// A SQLException was generated.  Catch it and display the error 
		// information.  Note that there could be multiple error objects 
		// chained together
		debug("*** SQLException caught ***");

		while (ex != null) {
			String exceptionString = "SQLState: " + ex.getSQLState ()
				+ "\nMessage:  " + ex.getMessage ()
				+ "\nVendor:   " + ex.getErrorCode ();
			debug(exceptionString);
			JOptionPane.showMessageDialog(parent,exceptionString,"SQL Exception",JOptionPane.ERROR_MESSAGE);
			ex = ex.getNextException();
		}
	}

	// The default handler for ClassNotFound exceptions in this project

	static public void defaultClassNotFoundExceptionHandler(java.awt.Frame parent, ClassNotFoundException ex) {
		// A SQLException was generated.  Catch it and display the error 
		// information.  Note that there could be multiple error objects 
		// chained together
		debug("*** ClassNotFoundException caught ***");

		String exceptionString = "Message:  " + ex.getMessage ();
		debug(exceptionString);
		JOptionPane.showMessageDialog(parent,exceptionString,"Class Not Found Exception",JOptionPane.ERROR_MESSAGE);
	}

	// Provide a structured output mechanism
	
	static public void debug(String debugMsg) {
		System.out.println("\n" + Global.tabName + "> " + debugMsg + "\n");
	}
}
