/* ////////////////////////////////////////////////////////////////////////////

// Project: Sharing data from a RDBMS with a Protege project using JDBC/ODBC.

// Developer: My Nguyen

/////////////////////////////////////////////////////////////////////////////*/

package DataGenieTab;

///////////////////////////////////////////////////////////////////////////////
// IMPORTS

// For the Swing UI Components
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

// For the JDBC
import java.sql.*;

// For Vector and Collections classes
import java.util.*;

// For the Protege PDK
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;


///////////////////////////////////////////////////////////////////////////////
// HeuristicImportDialog

// The HeuristicImportDialog class is used to do a "heuristic" import, by setting
// up instance pointers based on entity relationships in the data.  These 
// relationships are the foreign key relationships that the user defines and,
// in some cases, are automatic detected by the tab widget (if the information
// is available).

class HeuristicImportDlg extends JDialog implements ActionListener {

	// Constructor
	
	public HeuristicImportDlg(java.awt.Frame parent, Vector tables, ConnectInfo connectInfo, KnowledgeBase kb) {
		super(parent,"Heuristic Import",true);

		this.connectInfo = connectInfo;
		
		dsInfo = new DataSourceInfo(this.connectInfo);

		setSize(new Dimension(450,400));
		
		Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());

		dataTables1 = new JComboBox(tables);
		dataTables1.setEditable(false);
		dataTables1.setRenderer(new MyListCellRenderer());
		dataTables1.addActionListener(this);
		dataTables2 = new JComboBox(tables);
		dataTables2.setEditable(false);
		dataTables2.setRenderer(new MyListCellRenderer());
		dataTables2.addActionListener(this);		
		
		TreeNodeInfo nodeInfo = (TreeNodeInfo)dataTables1.getSelectedItem();
		String tableName = nodeInfo.m_text;
		dataCols1 = new JComboBox(getComboBoxModel(tableName));
		dataCols1.setEditable(false);
		dataCols1.setRenderer(new MyListCellRenderer());

		nodeInfo = (TreeNodeInfo)dataTables2.getSelectedItem();
		tableName = nodeInfo.m_text;
		dataCols2 = new JComboBox(getComboBoxModel(tableName));
		dataCols2.setEditable(false);
		dataCols2.setRenderer(new MyListCellRenderer());

		JPanel p1 = new JPanel(new GridLayout(2,2));
		p1.add(new LabeledComponent("Source Table",dataTables1,false));
		p1.add(new LabeledComponent("Target Table",dataTables2,false));
		p1.add(new LabeledComponent("Foreign Key",dataCols1,false));
		p1.add(new LabeledComponent("Primary Key",dataCols2,false));
		contentPane.add(p1,BorderLayout.NORTH);

		JPanel p2 = new JPanel();
		keysListModel = new DefaultListModel();
		keysListBox = new JList(keysListModel);
		JScrollPane keysScrollPane = new JScrollPane(keysListBox);
		p2.add(new LabeledComponent("Foreign Key Relationships",keysScrollPane,true));

		JPanel p3 = new JPanel(new FlowLayout());
		addButton = new JButton("Add");
		addButton.setMnemonic('A');
		addButton.addActionListener(this);
		p3.add(addButton);

		JPanel p4 = new JPanel(new BorderLayout());
		p4.add(p3,BorderLayout.NORTH);
		p4.add(p2,BorderLayout.CENTER);

		contentPane.add(p4,BorderLayout.CENTER);

		JPanel southPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));

		JButton detectButton = new JButton("Detect");
		detectButton.setMnemonic('D');
		detectButton.setEnabled(false);
		detectButton.addActionListener(this);
		southPanel.add(detectButton);

		delButton = new JButton("Remove");
		delButton.setMnemonic('R');
		delButton.setEnabled(false);
		delButton.addActionListener(this);
		southPanel.add(delButton);

		removeAllButton = new JButton("Remove All");
		removeAllButton.setMnemonic('l');
		removeAllButton.setEnabled(false);
		removeAllButton.addActionListener(this);
		southPanel.add(removeAllButton);

		finishButton = new JButton("Finish");
		finishButton.setMnemonic('F');
		finishButton.addActionListener(this);
		southPanel.add(finishButton);

		cancelButton = new JButton("Cancel");
		cancelButton.setMnemonic('C');
		cancelButton.addActionListener(this);
		southPanel.add(cancelButton);

		contentPane.add(southPanel,BorderLayout.SOUTH);

		this.kb = kb;
	}

	// Show the dialog to the user
	
    public boolean showDialog() {
		show();
		return ok;
	}

	// Handle action events
	
	public void actionPerformed(ActionEvent e) {
		Object src = e.getSource();
		if (src == cancelButton) {
			setVisible(false);
		} else if (src == dataTables1) {
			TreeNodeInfo nodeInfo = (TreeNodeInfo)dataTables1.getSelectedItem();
			String tableName = nodeInfo.m_text;
			dataCols1.setModel(getComboBoxModel(tableName));
		} else if (src == dataTables2) {
			TreeNodeInfo nodeInfo = (TreeNodeInfo)dataTables2.getSelectedItem();
			String tableName = nodeInfo.m_text;
			dataCols2.setModel(getComboBoxModel(tableName));
		} else if (src == addButton) {
			TreeNodeInfo nodeInfo = (TreeNodeInfo)dataTables1.getSelectedItem();
			String tableName1 = nodeInfo.m_text;
			nodeInfo = (TreeNodeInfo)dataTables2.getSelectedItem();
			String tableName2 = nodeInfo.m_text;
			TreeNodeInfo foreignKey = (TreeNodeInfo)dataCols1.getSelectedItem();
			TreeNodeInfo primaryKey = (TreeNodeInfo)dataCols2.getSelectedItem();
			if (foreignKey.m_colSQLType == primaryKey.m_colSQLType) {
				RelationshipKey newKey = new RelationshipKey(tableName1,foreignKey.m_text,tableName2,primaryKey.m_text);
				if (keysListModel.indexOf(newKey) == -1) {
					keysListModel.addElement(newKey);
					delButton.setEnabled(true);
					removeAllButton.setEnabled(true);
				} else {
					String msg = newKey.toString() + " is already defined.";
					JOptionPane.showMessageDialog(JOptionPane.getFrameForComponent(this),msg,Global.tabName,JOptionPane.ERROR_MESSAGE);
				}
			} else
				JOptionPane.showMessageDialog(JOptionPane.getFrameForComponent(this),"Type incompatible",Global.tabName,JOptionPane.ERROR_MESSAGE);
		} else if (src == delButton) {
			int idx = keysListBox.getSelectedIndex();
			if (idx != -1) {
				keysListModel.removeElementAt(idx);
				if (keysListModel.size() == 0) {
					delButton.setEnabled(false);
					removeAllButton.setEnabled(false);
				}
			}
		} else if (src == removeAllButton) {
			keysListModel.removeAllElements();
			delButton.setEnabled(false);
			removeAllButton.setEnabled(false);
		} else if (src == finishButton) {
			setVisible(false);
			ok = true;
		}
	}

	// Get a ComboBox model that contains the field names in 
	// the given table
	
	private DefaultComboBoxModel getComboBoxModel(String tableName) {
		Vector colNames = getColumnNames(tableName);
		return new DefaultComboBoxModel(colNames);
	}

	// Get the field names of the given table

	private Vector getColumnNames(String tableName) {
		Vector colNames = new Vector();
		try {
			Class.forName(connectInfo.m_driver);
    
			Connection con = DriverManager.getConnection(connectInfo.m_DSN, 
				connectInfo.m_UID, connectInfo.m_PWD);
        
			DatabaseMetaData dbMetaData = con.getMetaData();

			
			// Getting primary keys
			Vector primaryKeys = new Vector();
			if (dsInfo.m_supportsCoreSQLGrammar) {
				ResultSet rs = dbMetaData.getPrimaryKeys(null,null,tableName);
				boolean more = rs.next();
				Global.debug("Primary Keys Info:");
				while (more) {
					String info = rs.getString("TABLE_NAME");
					String colName = rs.getString("COLUMN_NAME");
					info = info
						+ ":" + colName
						+ " " + Short.toString(rs.getShort("KEY_SEQ"))
						+ " " + rs.getString("PK_NAME");
					Global.debug(info);
					primaryKeys.add(colName);
					more = rs.next();
				}
				rs.close();
			}

			// Test getting exported keys
			ResultSet rs = dbMetaData.getExportedKeys(null,null,tableName);
			boolean more = rs.next();
			Global.debug("Exported Keys Info:");
			while (more) {
				String pkTableName = rs.getString("PKTABLE_NAME");
				String pkColName = rs.getString("PKCOLUMN_NAME");
				String fkTableName = rs.getString("FKTABLE_NAME");
				String fkColName = rs.getString("FKCOLUMN_NAME");
				String info = fkTableName + ":" + fkColName + "->"
					+ pkTableName + ":" + pkColName;
				Global.debug(info);
				more = rs.next();
			}
			rs.close();

			// Test getting imported keys
			rs = dbMetaData.getImportedKeys(null,null,tableName);
			more = rs.next();
			Global.debug("Imported Keys Info:");
			while (more) {
				String pkTableName = rs.getString("PKTABLE_NAME");
				String pkColName = rs.getString("PKCOLUMN_NAME");
				String fkTableName = rs.getString("FKTABLE_NAME");
				String fkColName = rs.getString("FKCOLUMN_NAME");
				String info = fkTableName + ":" + fkColName + "->"
					+ pkTableName + ":" + pkColName;
				Global.debug(info);
				more = rs.next();
			}
			rs.close();

			ResultSet rsColumns = dbMetaData.getColumns(null,null,tableName,null);
			boolean bMore = rsColumns.next();
			while (bMore) {
				String colName = rsColumns.getString("COLUMN_NAME");
				int sqlType = rsColumns.getShort("DATA_TYPE");

				// Determine whether this column is a primary key
				int colType = TreeNodeInfo.COLUMN;
				for (int i = 0; i < primaryKeys.size(); i++) {
					String key = (String)primaryKeys.elementAt(i);
					if (key.compareToIgnoreCase(colName) == 0) {
						colType = TreeNodeInfo.COLUMN_PK;
						break;
					}
				}

				colNames.addElement(new TreeNodeInfo(colName,colType,sqlType));
				bMore = rsColumns.next();
			}

			rsColumns.close();
			con.close();
		} catch (SQLException ex) {
			Global.defaultSQLExceptionHandler(JOptionPane.getFrameForComponent(this),ex);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return colNames;
	}

	// Set the class data

	public void setNewData(Vector tables, ConnectInfo connectInfo) {
		dataTables1.removeAllItems();
		dataTables2.removeAllItems();
		dataCols1.removeAllItems();
		dataCols2.removeAllItems();
		this.connectInfo = connectInfo;

		Enumeration enum = tables.elements();
		while (enum.hasMoreElements()) {
			String strTable = (String)enum.nextElement();
			dataTables1.addItem(strTable);
			dataTables2.addItem(strTable);
		}
	}

	// Do a one-to-many or many-to-one import

	private void otommtoo(RelationshipKey key) {
		Cls cls = kb.getCls(key.strTable1);
		String slotName = key.strTable2+" Ptrs";
		Vector classes = new Vector();
		classes.add(kb.getCls(key.strTable2));

		Slot newSlot = kb.getSlot(slotName);
		if (newSlot == null) {
			newSlot = kb.createSlot(slotName);
			newSlot.setValueType(ValueType.INSTANCE);
			newSlot.setAllowsMultipleValues(false);
			newSlot.setAllowedClses(classes);
			cls.addDirectTemplateSlot(newSlot);
		} else {							
			if (newSlot.getValueType() == ValueType.INSTANCE
				&& newSlot.getAllowsMultipleValues() == false
				&& newSlot.getAllowedClses() == classes)
				cls.addDirectTemplateSlot(newSlot);
			else {								
				java.awt.Frame frame = JOptionPane.getFrameForComponent(this);
				NewSlotNameDialog dialog = new NewSlotNameDialog(frame,slotName);
				dialog.setLocationRelativeTo(JOptionPane.getFrameForComponent(this));
				while (true) {
					if (dialog.showDialog()) {
						newSlot = kb.getSlot(dialog.selectedSlotName);
						if (newSlot == null)
							break;
					}
				}
				newSlot = kb.createSlot(dialog.selectedSlotName);
				newSlot.setValueType(ValueType.INSTANCE);
				newSlot.setAllowsMultipleValues(false);
				newSlot.setAllowedClses(classes);
				cls.addDirectTemplateSlot(newSlot);
			}
		}

		Collection collection = cls.getDirectInstances();
		Iterator iterator = collection.iterator();
		while (iterator.hasNext()) {
			Instance inst = (Instance)iterator.next();
			Slot slot = kb.getSlot(key.strForeignKey);
			Object obj = inst.getOwnSlotValue(slot);
			
			Vector instPtrs = new Vector();

			Collection coll2 = (kb.getCls(key.strTable2)).getDirectInstances();
			Iterator iter2 = coll2.iterator();
			while (iter2.hasNext()) {
				Instance inst2 = (Instance)iter2.next();

				Slot slot2 = kb.getSlot(key.strPrimaryKey);
				Object obj2 = inst2.getOwnSlotValue(slot2);
				
				if (obj.equals(obj2)) {
					instPtrs.add(inst2);
				}
			}

			inst.setOwnSlotValues(newSlot,instPtrs);
		}
	}

	// The main function that sets up instance pointers
	// and cardinality

	public void setUpInstancePtrsMtoM() {
		Vector mtomList = new Vector();
		Vector notMtoMList = new Vector();
		Enumeration enum = keysListModel.elements();
		while (enum.hasMoreElements()) {
			RelationshipKey key = (RelationshipKey)enum.nextElement();
			Enumeration enum2 = keysListModel.elements();
			boolean mtomFound = false;
			while (enum2.hasMoreElements()) {
				RelationshipKey key2 = (RelationshipKey)enum2.nextElement();
				if (key != key2 && key.strTable1 == key2.strTable1) {
					if (notMtoMList.indexOf(key) != -1)
						break;
					if (mtomList.indexOf(key) == -1) {
						String msg = key.toString() + "\n" + key2.toString() + "\n\n"
							+ "Is this a many-to-many relationship?";
						int retVal = JOptionPane.showConfirmDialog(JOptionPane.getFrameForComponent(this),msg,Global.tabName,JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE);
						if (retVal == JOptionPane.YES_OPTION) {
							//Global.debug("YES");
							mtom(key, key2);
							mtomList.add(key2);
							mtomFound = true;
							break;
						} else {
							notMtoMList.add(key2);
							break;
						}
					} else
						mtomFound = true;
				}
			}

			if (mtomFound == false) {
				//Global.debug("one-to-one: " + key.toString());
				otommtoo(key);
			}				
		}
		kb.setChanged(true);
		String strMsg = "Heuristic Import Completed";
		JOptionPane.showMessageDialog(JOptionPane.getFrameForComponent(this),strMsg,Global.tabName,JOptionPane.INFORMATION_MESSAGE);
	}

	// Do a many-to-many import

	private void mtom(RelationshipKey key, RelationshipKey key2) {
		// add 1st slot to 1st table
		Cls cls = kb.getCls(key.strTable2);
		String slotName = key2.strTable2+" Ptrs";
		Vector newClasses = new Vector();
		newClasses.add(kb.getCls(key2.strTable2));

		Slot newSlot = kb.getSlot(slotName);
		if (newSlot == null) {
			newSlot = kb.createSlot(slotName);
			newSlot.setValueType(ValueType.INSTANCE);
			newSlot.setAllowsMultipleValues(true);
			newSlot.setAllowedClses(newClasses);
			cls.addDirectTemplateSlot(newSlot);
		} else {							
			if (newSlot.getValueType() == ValueType.INSTANCE
				&& newSlot.getAllowsMultipleValues() == true
				&& newSlot.getAllowedClses() == newClasses)
				cls.addDirectTemplateSlot(newSlot);
			else {								
				java.awt.Frame frame = JOptionPane.getFrameForComponent(this);
				NewSlotNameDialog dialog = new NewSlotNameDialog(frame,slotName);
				dialog.setLocationRelativeTo(JOptionPane.getFrameForComponent(this));
				while (true) {
					if (dialog.showDialog()) {
						newSlot = kb.getSlot(dialog.selectedSlotName);
						if (newSlot == null)
							break;
					}
				}
				newSlot = kb.createSlot(dialog.selectedSlotName);
				newSlot.setValueType(ValueType.INSTANCE);
				newSlot.setAllowsMultipleValues(true);
				newSlot.setAllowedClses(newClasses);
				cls.addDirectTemplateSlot(newSlot);
			}
		}

		// add 2nd slot to 2nd table
		Cls cls2 = kb.getCls(key2.strTable2);
		String slotName2 = key.strTable2+" Ptrs";
		Vector classes = new Vector();
		classes.add(kb.getCls(key.strTable2));

		Slot newSlot2 = kb.getSlot(slotName2);
		if (newSlot2 == null) {
			newSlot2 = kb.createSlot(slotName2);
			newSlot2.setValueType(ValueType.INSTANCE);
			newSlot2.setAllowsMultipleValues(true);
			newSlot2.setAllowedClses(classes);
			cls2.addDirectTemplateSlot(newSlot2);
		} else {							
			if (newSlot2.getValueType() == ValueType.INSTANCE
				&& newSlot2.getAllowsMultipleValues() == true
				&& newSlot2.getAllowedClses() == classes)
				cls2.addDirectTemplateSlot(newSlot2);
			else {								
				java.awt.Frame frame = JOptionPane.getFrameForComponent(this);
				NewSlotNameDialog dialog = new NewSlotNameDialog(frame,slotName2);
				dialog.setLocationRelativeTo(JOptionPane.getFrameForComponent(this));
				while (true) {
					if (dialog.showDialog()) {
						newSlot2 = kb.getSlot(dialog.selectedSlotName);
						if (newSlot2 == null)
							break;
					}
				}
				newSlot2 = kb.createSlot(dialog.selectedSlotName);
				newSlot2.setValueType(ValueType.INSTANCE);
				newSlot2.setAllowsMultipleValues(true);
				newSlot2.setAllowedClses(classes);
				cls2.addDirectTemplateSlot(newSlot2);
			}
		}

		// per instance add approach
		// instance ptrs
		Cls relationCls = kb.getCls(key.strTable1);
		Collection coll = relationCls.getDirectInstances();
		Iterator iter = coll.iterator();
		while (iter.hasNext()) {
			Instance inst = (Instance)iter.next();
			Slot slot = kb.getSlot(key.strForeignKey);
			Object obj = inst.getOwnSlotValue(slot);

			Vector instPtrsA = new Vector();
			Vector instPtrsB = new Vector();

			// find
			Collection colla = cls.getDirectInstances();
			Iterator itera = colla.iterator();
			while (itera.hasNext()) {
				Instance insta = (Instance)itera.next();

				Slot slota = kb.getSlot(key.strPrimaryKey);
				Object obja = insta.getOwnSlotValue(slota);
				
				if (obj.equals(obja)) {
					instPtrsA.add(insta);
					//Global.debug("add a");
				}
			}

			// add and find
			Slot slot2 = kb.getSlot(key2.strForeignKey);
			Object obj2 = inst.getOwnSlotValue(slot2);

			Collection collb = cls2.getDirectInstances();
			Iterator iterb = collb.iterator();
			while (iterb.hasNext()) {
				Instance instb = (Instance)iterb.next();

				Slot slotb = kb.getSlot(key2.strPrimaryKey);
				Object objb = instb.getOwnSlotValue(slotb);
				
				if (obj2.equals(objb)) {
					//instb.setOwnSlotValues(newSlot2,instPtrsA);
					int size = instPtrsA.size();
					for (int i = 0; i < size; i++)
						instb.addOwnSlotValue(newSlot2,instPtrsA.get(i));
					instPtrsB.add(instb);
				}
			}

			// add
			colla = cls.getDirectInstances();
			itera = colla.iterator();
			while (itera.hasNext()) {
				Instance insta = (Instance)itera.next();

				Slot slota = kb.getSlot(key.strPrimaryKey);
				Object obja = insta.getOwnSlotValue(slota);
				
				if (obj.equals(obja)) {
					//insta.setOwnSlotValues(newSlot,instPtrsB);
					int size = instPtrsB.size();
					for (int i = 0; i < size; i++)
						insta.addOwnSlotValue(newSlot,instPtrsB.get(i));
				}
			}
		}
	}

	// Class data

	private JButton addButton, delButton, removeAllButton;
	private JButton cancelButton, finishButton;

	private JComboBox dataTables1, dataTables2;
	private JComboBox dataCols1, dataCols2;
	
	private JList keysListBox;
	private DefaultListModel keysListModel;

	private ConnectInfo connectInfo;
    private KnowledgeBase kb;

	private boolean ok;

	private DataSourceInfo dsInfo;
}

