/* ////////////////////////////////////////////////////////////////////////////

// Project: Sharing data from a RDBMS with a Protege project using JDBC/ODBC.

// Developer: My Nguyen

/////////////////////////////////////////////////////////////////////////////*/

package DataGenieTab;

///////////////////////////////////////////////////////////////////////////////
// IMPORTS

// For the Swing UI Components
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

// For the JDBC
import java.sql.*;

// For Vector and Collections classes
import java.util.*;

// For the Protege PDK
import edu.stanford.smi.protege.model.*;


///////////////////////////////////////////////////////////////////////////////
// JDBCDataTable

// The JDBCDataTable class is a Table model that provides data to a JTable

class JDBCDataTable extends AbstractTableModel {

    // A default constructor that does nothing at the moment
	
	public JDBCDataTable() { 
    }
    
	// Execute a query to obtain data for the model

    public boolean executeQuery(ConnectInfo connectInfo, String strQuery, long numRows) {
        try {
            Class.forName(connectInfo.m_driver);
            con = DriverManager.getConnection(connectInfo.m_DSN, connectInfo.m_UID, connectInfo.m_PWD);
            stmt = con.createStatement();
            resultSet = stmt.executeQuery(strQuery);
            
            rsMetaData = resultSet.getMetaData();
            
            int numColumns = rsMetaData.getColumnCount();
            columnNames = new String[numColumns];
            for (int i = 0; i < numColumns; i++)
                columnNames[i] = rsMetaData.getColumnLabel(i+1);
                
            rows.removeAllElements();
			int count = 0;
            while (resultSet.next()) {
                Vector newRow = new Vector();
                for (int i = 0; i < numColumns; i++) {
                    int type = rsMetaData.getColumnType(i+1);
                    Object obj = resultSet.getObject(i+1);
//System.out.println("type: " + type + "(" + rsMetaData.getColumnTypeName(i+1) + ")");
//System.out.println("Obj: " + obj);
					newRow.addElement(Global.getSQLProtegeObject(type,obj));
                }
                rows.addElement(newRow);
				count++;
				if (count == numRows)
					break;
            }
            resultSet.close();
            stmt.close();
            con.close();
            
            fireTableChanged(null);
            return true;
        } catch (SQLException ex) {
            //System.out.println("error: not a table");
			//Global.defaultSQLExceptionHandler(ex);
            return false;
        } catch (ClassNotFoundException ex) {
            Global.debug("Error: Driver not found.");
			//ex.printStackTrace();
            return false;
        }
    }
    
    // Implementation of TableModel interface
    
    // See the Java Doc for additional information on the methods that follow
	
	// Return the name of a column
	
	public String getColumnName(int columnIndex) {
        if (columnNames[columnIndex] != null)
            return columnNames[columnIndex];
        else
            return "";
    }

	// Return the class type of a column
    
    public Class getColumnClass(int columnIndex) {
        int type;
        try {
            type = rsMetaData.getColumnType(columnIndex+1);
        } catch (SQLException e) {
            return super.getColumnClass(columnIndex);
        }   

        ValueType protegeType = Global.getSQLProtegeType(type);
		return protegeType.getJavaType();
    }

	// Return the number of rows in this model
    
    public int getRowCount() {
        return rows.size();
    }

	// Return the number of columns contained in this model
    
    public int getColumnCount() {
        return columnNames.length;
    }

	// Return the data residing at a row and column
    
    public Object getValueAt(int rowIndex, int columnIndex) {
        Vector row = (Vector)rows.elementAt(rowIndex);
        return row.elementAt(columnIndex);
    }

	// Class data
    
    private Connection con;
    private Statement stmt;
    private ResultSet resultSet;
    private ResultSetMetaData rsMetaData;
    
    private String[] columnNames = {};
    private Vector rows = new Vector();
}

